package org.bitbucket.bradleysmithllc.etlunit.feature.database;

import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;
import org.bitbucket.bradleysmithllc.etlunit.feature.file.FileRuntimeSupport;
import org.bitbucket.bradleysmithllc.etlunit.io.FileBuilder;
import org.bitbucket.bradleysmithllc.etlunit.io.file.DataFileManager;
import org.bitbucket.bradleysmithllc.etlunit.io.file.DataFileSchema;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestValueObject;

import javax.inject.Inject;
import java.io.File;
import java.io.IOException;

public class DatabaseRuntimeSupportImpl implements DatabaseRuntimeSupport
{
	private RuntimeSupport runtimeSupport;
	private FileRuntimeSupport fileRuntimeSupport;
	private DataFileManager dataFileManager;

	@Inject
	public void setRuntimeSupport(RuntimeSupport runtimeSupport)
	{
		this.runtimeSupport = runtimeSupport;
	}

	@Inject
	public void receiveFileRuntimeSupport(FileRuntimeSupport manager)
	{
		fileRuntimeSupport = manager;
	}

	@Inject
	public void receiveDataFileManager(DataFileManager manager)
	{
		dataFileManager = manager;
	}

	@Override
	public File getSourceDataDirectoryForCurrentTest()
	{
		return getSourceDataDirectory(runtimeSupport.getCurrentlyProcessingTestPackage());
	}

	@Override
	public File getSourceScriptsDirectoryForCurrentTest()
	{
		return getSourceScriptsDirectory(runtimeSupport.getCurrentlyProcessingTestPackage());
	}

	@Override
	public File getSourceDataSetForCurrentTest(String dataSetName)
	{
		return getSourceDataSet(runtimeSupport.getCurrentlyProcessingTestPackage(), dataSetName);
	}

	@Override
	public File getSourceScriptForCurrentTest(String scriptName)
	{
		return getSourceScript(runtimeSupport.getCurrentlyProcessingTestPackage(), scriptName);
	}

	public File getSourceDataDirectory(String package_)
	{
		return new FileBuilder(runtimeSupport.getTestResourceDirectory(package_, "data")).mkdirs().file();
	}

	public File getSourceScriptsDirectory(String package_)
	{
		return new FileBuilder(runtimeSupport.getTestResourceDirectory(package_, "sql")).mkdirs().file();
	}

	public File getSourceDataSet(String package_, String dataSetName)
	{
		return new FileBuilder(getSourceDataDirectory(package_)).name(dataSetName + ".delimited").file();
	}

	public File getSourceScript(String package_, String scriptName)
	{
		return new FileBuilder(getSourceScriptsDirectory(package_)).name(scriptName).file();
	}

	public File getGeneratedSourceDirectory(DatabaseConnection conn, String mode)
	{
		File dir = new FileBuilder(runtimeSupport.getGeneratedSourceDirectory("database")).subdir(conn.getId()).subdir(conn.getDatabaseName(mode)).mkdirs().file();

		return dir;
	}

	public File getGeneratedDataSet(DatabaseConnection conn, String mode, String dataSetName)
	{
		File dir = new FileBuilder(getGeneratedSourceDirectory(conn, mode)).name(dataSetName + ".delimited").file();

		return dir;
	}

	public DataFileSchema createRelationalMetaInfo(DatabaseConnection dbConn, String schema, String name)
	{
		return createRelationalMetaInfo(dbConn.getId(), schema, name);
	}

	private String createSchemaKey(String connId, String schema, String name)
	{
		return connId + "-" + schema + "-" + name;
	}

	public DataFileSchema createRelationalMetaInfo(String connId, String schema, String name)
	{
		return dataFileManager.createDataFileSchema(createSchemaKey(connId, schema, name));
	}

	public DataFileSchema getRelationalMetaInfo(DatabaseConnection dbConn, String schema, String name, String masterName, ETLTestValueObject etlObj)
	{
		return getRelationalMetaInfo(dbConn.getId(), schema, name, masterName, etlObj);
	}

	public DataFileSchema getRelationalMetaInfo(String connId, String schema, String name, String masterName, ETLTestValueObject etlObj)
	{
		return fileRuntimeSupport.locateReferenceFileSchema(masterName, createSchemaKey(connId, schema, name), etlObj);
	}

	public void persistDataFileSchema(DataFileSchema schema) throws IOException
	{
		fileRuntimeSupport.persistGeneratedDataFileSchema(schema);
	}
}